import { splitByString, splitByPattern } from './split.js';
function trim(value) {
    return value.trim();
}
function isNonEmptyString(value) {
    return value.length > 0;
}
function splitLines(value, lineSeparator = '\n') {
    return splitByString(value, lineSeparator).map(trim).filter(isNonEmptyString);
}
export function createGitCommandRunner(dependencies) {
    const { execute } = dependencies;
    return {
        async getShortStatus() {
            const result = await execute('git status --short');
            return result.stdout.trim();
        },
        async getCurrentBranchName() {
            const result = await execute('git rev-parse --abbrev-ref HEAD');
            return result.stdout.trim();
        },
        async fetchRemote(remoteAlias) {
            await execute(`git fetch ${remoteAlias}`);
        },
        async getSymmetricDifferencesBetweenBranches(branchA, branchB) {
            const result = await execute(`git rev-list --left-right ${branchA}...${branchB}`);
            return splitLines(result.stdout);
        },
        async getRemoteAliases() {
            const result = await execute('git remote -v');
            return splitLines(result.stdout).map((line) => {
                const remoteLineTokens = splitByPattern(line, /\s/);
                const [alias, url] = remoteLineTokens;
                if (url === undefined) {
                    throw new TypeError('Failed to determine git remote alias');
                }
                return { alias, url };
            });
        },
        async listTags() {
            const result = await execute('git tag --list');
            return splitLines(result.stdout);
        },
        async getMergeCommitLogs(from) {
            const subjectPlaceholder = '%s';
            const bodyPlaceholder = '%b';
            const fieldSeperator = '__||__';
            const lineSeperator = '##$$@@$$##';
            const fields = [subjectPlaceholder, bodyPlaceholder];
            const format = `${fields.join(fieldSeperator)}${lineSeperator}`;
            const result = await execute(`git log --no-color --pretty=format:${format} --merges ${from}..HEAD`);
            const logs = splitLines(result.stdout, lineSeperator);
            return logs.map((log) => {
                const parts = splitByString(log, fieldSeperator);
                const [subject, body] = parts;
                return { subject, body: body === '' ? undefined : body };
            });
        }
    };
}
//# sourceMappingURL=git-command-runner.js.map